// FOToolBar.cpp : implementation file
//

#include "stdafx.h"
#include "FOToolBar.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFlyout - implemented only in this cpp file

class CFlyout : public CToolBar
{
// Construction
public:
	CFlyout(CFOToolBar* m_pToolBar) : m_pToolBar(m_pToolBar), index(-1) { }

	pair<UINT, UINT> getChosenButton() {	// returns the <index of button, command ID of button>
		if (index < 0) return make_pair(~0, ~0);
		TBBUTTON m_btnInfo;
		GetToolBarCtrl().GetButton(index, &m_btnInfo);
		return make_pair(index, m_btnInfo.idCommand);
	}

// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CFlyout)
	CSize CalcDynamicLayout(int nLength, DWORD dwMode) {
		CFrameWnd* pFrame = GetParentFrame();
		pFrame->ModifyStyle(WS_CAPTION, 0); // This line removes the caption

		return (CToolBar::CalcDynamicLayout(nLength, dwMode));
	}
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CFlyout() { }

	// Generated message map functions
protected:
	CFOToolBar* m_pToolBar;
	int index;

	//{{AFX_MSG(CFlyout)
		// NOTE - the ClassWizard will add and remove member functions here.
		afx_msg void OnMouseMove( UINT nFlags, CPoint point );
		afx_msg void OnLButtonUp( UINT nFlags, CPoint point );
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};

BEGIN_MESSAGE_MAP(CFlyout, CToolBar)
	//{{AFX_MSG_MAP(CFlyout)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		ON_WM_MOUSEMOVE()
		ON_WM_LBUTTONUP()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CFlyout::OnMouseMove(UINT nFlags, CPoint point)
{
	CToolBar::OnMouseMove(nFlags, point);
	
	CToolBarCtrl& m_toolbarCtrl = GetToolBarCtrl();
	int currentIndex = m_toolbarCtrl.HitTest(&point);

	// if we're clicking on the same button, just leave it checked and quit...
	if (currentIndex == index) return;

	if (currentIndex != index && index >= 0) {
		// uncheck the previous button
		TBBUTTON m_btnInfo;
		m_toolbarCtrl.GetButton(index, &m_btnInfo);
		m_toolbarCtrl.CheckButton(m_btnInfo.idCommand, FALSE);
	}

	if (currentIndex >= m_toolbarCtrl.GetButtonCount()) currentIndex = -1;

	index = currentIndex;
	if (index < 0) return;	// we're not clicking on any button - so quit

	// check the new button
	TBBUTTON m_btnInfo;
	m_toolbarCtrl.GetButton(index, &m_btnInfo);

	m_toolbarCtrl.CheckButton(m_btnInfo.idCommand);
}

void CFlyout::OnLButtonUp(UINT nFlags, CPoint point)
{
	// we handle this message in case the user switched a window in the middle of picking
	// a button... when returning, we'll have mouse focus on this flyout (in case the left button
	// wasn't released in the other window) and not on the parent toolbar. Therefore we want
	// everything to be regular when the button is up so we simply send the message to the
	// parent toolbar
	CToolBar::OnLButtonUp(nFlags, point);

	ClientToScreen(&point);
	m_pToolBar->ScreenToClient(&point);
	m_pToolBar->SendMessage(WM_LBUTTONUP, WPARAM(nFlags), MAKELPARAM(point.x, point.y));
}

/////////////////////////////////////////////////////////////////////////////
// CFOToolBar

CFOToolBar::CFOToolBar() : toDrawDropDownArrows(false), index(-1), nID(0)
{
}

CFOToolBar::~CFOToolBar()
{
}


BEGIN_MESSAGE_MAP(CFOToolBar, CToolBar)
	//{{AFX_MSG_MAP(CFOToolBar)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		ON_WM_LBUTTONDOWN()
		ON_WM_LBUTTONUP()
		ON_WM_MOUSEMOVE()
		ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFOToolBar message handlers

void CFOToolBar::addFlyout(UINT buttonIndex, UINT toolbarID)
{
	CToolBarCtrl& m_toolBarCtrl = GetToolBarCtrl();

	// get the information about the button. If there's no such button - return
	TBBUTTON m_btnInfo;
	if (m_toolBarCtrl.GetButton(buttonIndex, &m_btnInfo) == 0) return;

	// add the button & toolbar IDs to the ID list
	ids.push_back(make_pair(m_btnInfo.idCommand, toolbarID));

	// signal that a new drop-down arrow has to be drawn on the button
	toDrawDropDownArrows = true;
}

void CFOToolBar::OnLButtonDown(UINT nFlags, CPoint point)
{
	CToolBar::OnLButtonDown(nFlags, point);

	CToolBarCtrl& m_toolBarCtrl = GetToolBarCtrl();

	// test if we're inside a button
	index = m_toolBarCtrl.HitTest(&point);
	if (index < 0) return;	// not on a button

	TBBUTTON m_btnInfo;
	m_toolBarCtrl.GetButton(index, &m_btnInfo);

	// check if we attached a fly-out toolbar to this button
	for (vector<pair<UINT, UINT> >::const_iterator i = ids.begin(); i != ids.end(); ++i)
		if (i->first == m_btnInfo.idCommand) break;

	if (i == ids.end()) return;	// no flyout toolbar is attached to this button - so quit

	nID = i->second;

	// there's a flyout toolbar attached to this button, so start the timer..
	SetTimer(1, 500, NULL);
}

void CFOToolBar::OnLButtonUp(UINT nFlags, CPoint point)
{
	KillTimer(1);

	if (m_flyout.get()) {
		CToolBarCtrl& m_toolBarCtrl = GetToolBarCtrl();

		pair<UINT, UINT> m_chosenButton = m_flyout->getChosenButton();
		if (m_chosenButton.first == ~0) goto shutDownFlyout;

		TBBUTTON m_btnInfo;
		m_toolBarCtrl.GetButton(index, &m_btnInfo);

		// if the user chose a new option, display the new icon and update the command id
		for (vector<pair<UINT, UINT> >::iterator i = ids.begin(); i != ids.end(); ++i)
			if (i->first == m_btnInfo.idCommand) break;

		m_toolBarCtrl.GetImageList()->Replace(index, m_flyout->GetToolBarCtrl().GetImageList()->ExtractIcon(m_chosenButton.first));
		m_toolBarCtrl.SetCmdID(index, m_chosenButton.second);
		drawDropDownArrow(index);

		// now update our id lists so that when the user clicks on the same button next time,
		// the flyout toolbar will open even though its command ID has changed
		i->first = m_chosenButton.second;
		InvalidateRect(NULL, FALSE);
	}

shutDownFlyout:	
	m_flyout = auto_ptr<CFlyout>();

	CToolBar::OnLButtonUp(nFlags, point);
}

void CFOToolBar::OnMouseMove(UINT nFlags, CPoint point)
{
	CToolBar::OnMouseMove(nFlags, point);

	if (m_flyout.get()) {
		// if we have a flyout, delegate the messages to the flyout toolbar
		m_flyout->SetFocus();
		ClientToScreen(&point);
		m_flyout->ScreenToClient(&point);
		m_flyout->SendMessage(WM_MOUSEMOVE, WPARAM(nFlags), MAKELPARAM(point.x, point.y));
	}
}

void CFOToolBar::OnTimer(UINT nIDEvent)
{
	if (nIDEvent != 1) { CToolBar::OnTimer(nIDEvent); return; }

	m_flyout = auto_ptr<CFlyout>();

	// if we've reached here it means we clicked on the button long enough
	KillTimer(1);

	// create the toolbar
	m_flyout = auto_ptr<CFlyout>(new CFlyout(this));

	CFrameWnd* m_pFrameWnd = IsFloating() ? dynamic_cast<CFrameWnd*>(GetParentFrame()->GetParent()) : dynamic_cast<CFrameWnd*>(GetParentFrame());
	if (!m_flyout->CreateEx(m_pFrameWnd, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE
		| CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_flyout->LoadToolBar(nID)) { m_flyout = auto_ptr<CFlyout>(); return; }

	// calculate the coordinates where the flyout will be displayed - this should be right next
	// to the button
	CRect m_btnRect;
	GetToolBarCtrl().GetItemRect(index, m_btnRect);
	ClientToScreen(m_btnRect);

	CRect m_tbRect;
	GetWindowRect(m_tbRect);

	CPoint point(0, 0);

	if (IsFloating() || (GetBarStyle() & CBRS_ORIENT_HORZ)) {
		// if floating - the flyout is displayed right under the button
		point.x = m_btnRect.left;
		point.y = m_btnRect.bottom;
	}
	else {
		// vertical - so the flyout is displayed to the right of the button
		point.x = m_btnRect.right;
		point.y = m_btnRect.top;
	}

	m_flyout->EnableDocking(CBRS_ALIGN_ANY);
	m_pFrameWnd->FloatControlBar(m_flyout.get(), point);

	m_flyout->SetFocus();
}

void CFOToolBar::drawDropDownArrow(int btnIndex)
{
	CImageList* m_imageList = GetToolBarCtrl().GetImageList();

	// extract the button's icon and get its HBITMAP
	HICON hIcon = m_imageList->ExtractIcon(btnIndex);
	ICONINFO m_iconInfo; GetIconInfo(hIcon, &m_iconInfo);
	BITMAP m_bmpInfo; GetObject(m_iconInfo.hbmColor, sizeof(m_bmpInfo), &m_bmpInfo);

	// create a memory device context to draw into
	CDC m_dc; m_dc.CreateCompatibleDC(NULL);

	// prepare the mask
	CBitmap m_mask; m_mask.CreateBitmapIndirect(&m_bmpInfo);

	CBitmap* m_pOldBitmap = (CBitmap*) m_dc.SelectObject(&m_mask);

	CBrush m_brush; m_brush.CreateSolidBrush(RGB(0, 0, 0));
	CBrush* m_pOldBrush = (CBrush*) m_dc.SelectObject(&m_brush);

	// draw the mask
	m_imageList->Draw(&m_dc, btnIndex, CPoint(0, 0), ILD_MASK);

	// draw the arrow at the right-bottom side of the bitmap
	POINT points[] = {	{m_bmpInfo.bmWidth-4, m_bmpInfo.bmHeight-1},
						{m_bmpInfo.bmWidth-1, m_bmpInfo.bmHeight-1},
						{m_bmpInfo.bmWidth-1, m_bmpInfo.bmHeight-4} };

	m_dc.Polygon(points, 3);
	m_dc.FloodFill(m_bmpInfo.bmWidth-3, m_bmpInfo.bmHeight-2, RGB(0, 0, 0));

	m_dc.SelectObject(m_pOldBrush); m_brush.DeleteObject();
	m_dc.SelectObject(m_pOldBitmap);

	// draw the arrow the image bitmap
	CBitmap m_bitmap; m_bitmap.Attach(m_iconInfo.hbmColor);

	m_pOldBitmap = (CBitmap*) m_dc.SelectObject(&m_bitmap);

	static COLORREF arrowColor = RGB(255, 0, 0);

	m_brush.CreateSolidBrush(arrowColor);
	m_pOldBrush = (CBrush*) m_dc.SelectObject(&m_brush);

	CPen m_pen; m_pen.CreatePen(PS_SOLID, 1, arrowColor);
	CPen* m_pOldPen = (CPen*) m_dc.SelectObject(&m_pen);

	m_dc.Polygon(points, 3);
	m_dc.FloodFill(m_bmpInfo.bmWidth-3, m_bmpInfo.bmHeight-2, arrowColor);

	m_dc.SelectObject(m_pOldBrush);
	m_dc.SelectObject(m_pOldBitmap);
	m_dc.SelectObject(m_pOldPen);

	// replace the bitmap with the new bitmap (the one with the arrow)
	m_imageList->Replace(btnIndex, &m_bitmap, &m_mask);

	m_bitmap.DeleteObject();
	m_pen.DeleteObject();
	m_brush.DeleteObject();
	m_mask.DeleteObject();

	m_dc.DeleteDC();
}

